function EnergyCompact_DFT_DCT_Walsh_Haar_2D_CRC;

% Energy Compaction capability of DFT, DCT, Haar and Walsh-Hadamard bases 
% for image of 2^n samples
% Displays selected image,DFT,DCT,Walsh and Haar spectra
% and graphs of spectra energy as function of bandwidth of low pass filter
% 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Usage EnergyCompact_DFT_DCT_Walsh_Haar_2D_CRC;

s=1;
while s==1;
p=menu('Select a test image','Piece-wise constant image','"Cameraman"','"Text"','Angiogram','Air photograph','Chest X-ray');
switch p,
    case 1, load pw; INPIMG=pw;
    case 2, load cameraman; INPIMG=cameraman;
    case 3, load text256; INPIMG=text256;
    case 4, load ango; INPIMG=ango;
    case 5, load mosc; INPIMG=mosc;
    case 6, load flu; INPIMG=flu;
end
INPIMG=INPIMG-mean(INPIMG(:));
[SzX,Szy]=size(INPIMG);
n=log(SzX)/log(2);
wal=walsh(n);
har=haar(n);
close all;
figure_n(6);
N=SzX;

%----------Spectra----------------------------
%----------DFT--------------------------------
sp_dft_lp=zeros(1,N/2);
sp_dft=abs(fft2(INPIMG)).^2;
sp_dft_norm=sp_dft/(sum(abs(sp_dft(:))));
for L=1:N/2,
    sp_dft_lp(L)=2*sum(sp_dft_norm(2:L-1,1))+sp_dft_norm(L,1)+.......
                 2*sum(sp_dft_norm(1,2:L-1))+sp_dft_norm(1,L)+.......
                 2*(sum(sum(sp_dft_norm(2:L,2:L))))+......
                 2*(sum(sum(sp_dft_norm(2:L,N-L+2:N))));
end

%-----------DCT-----------------------------
sp_dct_lp=zeros(1,SzX);
sp_dct=dct2(INPIMG);
sp_dct_norm=sp_dct/(sqrt(sum(abs(sp_dct(:)).^2)));
for L=1:N,
    Lsum_1=1:L;
    sp_dct_lp(L)=sum(sum(abs(sp_dct_norm(Lsum_1,Lsum_1)).^2));
end
plot(sp_dct_lp);
    
 %------------Walsh-------------------------  
sp_walsh_lp=zeros(1,SzX);
sp_walsh=abs(wal*INPIMG*wal).^2;
sp_walsh_norm=sp_walsh/(sum(sp_walsh(:)));
for L=1:N,
    Lsum_1=1:L;
    sp_walsh_lp(L)=sum(sum(abs(sp_walsh_norm(Lsum_1,Lsum_1))));
end
  
 %------------Haar-----------------------------
sp_haar_lp=zeros(1,SzX);
sp_haar=(abs(har*(har*INPIMG)').^2)';
sp_haar_norm=sp_haar/(sum(sp_haar(:)));
for L=1:N,
    Lsum_1=1:L;
    sp_haar_lp(L)=sum(sum(abs(sp_haar_norm(Lsum_1,Lsum_1))));
end
%-------Display and plot---------------------------
figure(1);
display1(INPIMG);
title('Test image','FontWeight','Bold','FontSize',12);

figure(2);
display1(fftshift(abs(sp_dft).^0.15));
title('DFT spectrum of the image','FontWeight','Bold','FontSize',12);
    
figure(3);
display1(abs(sp_dct(SzX:-1:1,:)).^0.15);
title('DCT spectrum of the image','FontWeight','Bold','FontSize',12);
    
figure(4);
display1(sp_walsh(SzX:-1:1,:).^0.15);
title('Walsh spectrum of the image','FontWeight','Bold','FontSize',12);
    
figure(5);
display1(sp_haar(SzX:-1:1,:).^0.15);
title('Haar spectrum of the image','FontWeight','Bold','FontSize',12 );

figure(6)
plot((2:2:SzX)/SzX,sp_dct_lp(2:2:SzX),'k-','LineWidth',2);
% 	axis([0.1 1 0.8 1]);
hold on
plot((1:SzX/2)/(SzX/2),sp_dft_lp,'c-','LineWidth',2);
plot((2:2:SzX)/SzX,sp_walsh_lp(2:2:SzX),'r-','LineWidth',2);
plot((2:2:SzX)/SzX,sp_haar_lp(2:2:SzX),'b-','LineWidth',2);
grid on
title('Fraction of signal energy','FontWeight','Bold','FontSize',12)
xlabel('Bandwidth (in fraction of the base band)','FontWeight','Bold','FontSize',12);
set(gca,'FontSize',12,'FontWeight','Bold')
legend('DCT','DFT','Walsh','Haar',4 );
axis tight
hold off

s=menu('Repeat with another image?','Yes','No');
end
close all
%-----------------------------------------------------------------------
function display1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display1(INPIMG)

clf
colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;

%------------------------------------------------
function WAL=walsh(n);

% Matrix of Walsh Hadamard transform of the order 2^n
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call WAL=walsh(n);

Sz=2^n;
WAL=hadamard(Sz);
gray=grayperm(n);
bin=bininv(n);
WAL=gray*bin*WAL;

%---------------------------------------------------

function BINV=bininv(n);

% Binary inversion permutation matrix of the order 2^n
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call  BINV=bininv(n);

BINV=eye(2);

for k=1:n-1,
	BINV_0=kron(BINV,[1 0]);
	BINV_1=kron(BINV,[0 1]);
	BINV=[BINV_0;BINV_1];
end

%-------------------------------------
function OUT=haar(n);

% Generating Haar matrix of the order 2^n.
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUT=haar(n);

E0=[1 1];
E1=[1 -1];
har_0=kron_n(E0,n-1);
OUT=kron_n(E0,n);
OUT=[OUT;kron(E1,har_0)];

for i=2:n,
	har_0=kron_n(E0,n-i);
	har_i=kron(eye(2^(i-1)),E1);
	har_i=kron(har_i,har_0);
	OUT=[OUT;har_i];
end

norm=inv(sqrt(OUT*OUT'));
OUT=norm*OUT;

%-------------------------------------------------
%--------------------------------------------------
function GRAY=grayperm(n)

% Gray-to-Direct Binary Code permutation matrix of the order 2^n
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call GRAY=grayperm(n)

I2=eye(2); 
I2_=[0 1;1 0];
	I_n=kron_n(I2,(n-1));
	I__n=kron_n(I2_,(n-1));
	GRAY=dir_sum(I_n,I__n);
	

for r=1:n-1,
	I2_r=kron_n(I2,r);
	I_r=kron_n(I2,(n-r-1));
	I__r=kron_n(I2_,(n-r-1));
	I=dir_sum(I_r,I__r);
	GRAY=GRAY*kron(I2_r,I);
end
%---------------------------------------------
function OUT=kron_n(INP,n);

% n-th Kronecker power of matrix INP
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUT=kron_n(INP,n); 

OUT=INP;
if n==0, OUT=1;
else
for k=1:n-1,
    OUT=kron(OUT,INP);
end
end
%----------------------------------
function DIRSUM=dir_sum(M1,M2);

% Direct sum of matrices M1 and M2
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call DIRSUM=dir_sum(M1,M2);

[SzX1 SzY1]=size(M1);
[SzX2 SzY2]=size(M2);

DIRSUM=[M1 zeros(SzX1,SzY2);zeros(SzX2,SzY1) M2];
%-------------------------------------------------
%-----------------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
   
%------------------------------------------------------


