function out=neighborhoods_CRC;
% Interactive selection and observation of pixel neighborhoods
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call  out=neighborhoods_CRC;;


figure_n(1);
m=menu('Select image','Angiogram','Bacteria');
switch m,
    case 1, load angio; IMG=angio;
    case 2, load bacteria; IMG=bacteria;
end


IMG=double(IMG);
[SzX,SzY]=size(IMG);
imshow(IMG,[],'InitialMagnification','fit');
title('Enter coordinates of the center of the window','FontWeight','Bold');drawnow;

t='Enter coordinates of the center of the window';
[Y0,X0]=ginput(1); 
Y0=round(Y0);
X0=round(X0);
WSZ=inputdlg({'Fragment vertical size 2Lx+1;, Lx=','Fragment horizontal size 2Ly+1; Ly='},'Enter fragment size',1,{'7','7'});
Lx=str2double(WSZ(1));
Ly=str2double(WSZ(2));
Xmin=X0-Lx;if Xmin<1,Xmin=1;end;Xmax=X0+Lx;if Xmax>SzX,Xmax=SzX;end
Ymin=Y0-Ly;if Ymin<1,Ymin=1;end;Ymax=Y0+Ly;if Ymax>SzY,Ymax=SzY;end
eye=IMG(Xmin:Xmax,Ymin:Ymax);
% WSz=size(eye);
[WSzX,WSzY]=size(eye);
WSz=WSzX*WSzY;
dsplmask=ones(size(IMG));
dsplmask(Xmin:Xmax,Ymin:Ymax)=1.5*dsplmask(Xmin:Xmax,Ymin:Ymax);
subplot(2,2,1);
imshow(IMG.*dsplmask,[]);
title('Input image and selected fragment','FontWeight','Bold');
drawnow;

centervalue=eye(Lx+1,Ly+1);
h=imghisto(eye);
h_cum=cumsum(h);
h0=zeros(1,256);h0(centervalue)=max(h)/2;

subplot(2,2,2);imshow(eye,[]);
title(['Selected fragment ',num2str(Xmax-Xmin+1),'x',num2str(Ymax-Ymin+1)],'FontWeight','Bold');drawnow;
subplot(2,2,3);
plot(1:256,h,'k-',1:256,h0,'b--','LineWidth',2);
grid on;
title('Histogram of the selected fragment','FontWeight','Bold');
xlabel('Gray level','FontWeight','Bold');
axis([min(eye(:)) max(eye(:)) 0 max(h)]);
set(gca','FontWeight','Bold');
drawnow;

s=1;
while s==1,
    mn=menu('Select neighborhood','EV-neigborhood','KNV-neighborhood','ER-neighborhood',.......
    'Q-neighborhood','CL-neighborhood');
    switch mn,
%------------------------EV-neighborhood-----------------------------------
    case 1,
    s=1;
    while s==1,
        EV=inputdlg({'EVplus','EVminus'},'Enter EV',1,{'10','10'});
        evpos=str2double(EV(1));
        evneg=str2double(EV(2));
        eye(Lx+1,Ly+1)=centervalue;
        mask=(eye<=(centervalue+evpos)).*(eye>=(centervalue-evneg));
        out=mask;
        h1=nan(1,256);h1(centervalue-evneg)=max(h);
        h2=nan(1,256);h2(centervalue+evpos)=max(h);
        subplot(2,2,3);
        plot(1:256,h,'k-',1:256,h0,'b--','LineWidth',2);
        title('Fragment histogram and Ev-nbh borders','FontWeight','Bold');
        axis([min(eye(:))-evneg max(eye(:))+evpos 0 max(h)]);
        grid on
        xlabel('Gray level','FontWeight','Bold');
        set(gca','FontWeight','Bold')
        hold on
        stem(1:256,h1,'r-->','LineWidth',2);
        stem(1:256,h2,'r--<','LineWidth',2);
        hold off
        subplot(2,2,4);
        imshow(out.*eye,[]);
        title(['EV-neighborhood; EV_+=' int2str(evpos) '; EV_-=' int2str(evneg)],'FontWeight','Bold');
        drawnow;
        s=menu('Other EVs','Yes','No'); 
    end
    %----------------------------KNV-neighborhood--------------------------
    case 2
    s=1;
    while s==1,
        k=inputdlg({['K in fraction of the fragment size',num2str(WSzX),'x',num2str(WSzY)]},'Enter K',1,{'0.5'});
        k=str2double(k);
        K=round(WSzX*WSzY*k);
        MN=min(eye(:));
        MX=max(eye(:));
        Vd=centervalue;
        Vu=centervalue;
        while k<=K,
            Vd=Vd-1;if Vd<MN,Vd=MN;end
            Vu=Vu+1;if Vu>MX,Vu=MX;end;
            mask=(eye>=Vd).*(eye<=Vu);
            k=sum(mask(:));
        end
        out=mask.*eye;
        out_sort=sort(out(:));
        KSN_min=out_sort(WSz-K+1);
        KSN_max=out_sort(WSz);
        h_r=zeros(1,256);
        h_r(centervalue)=h_cum(centervalue);
        h_r_min=nan(1,256);
        h_r_min(KSN_min)=h_cum(KSN_min);
        h_r_max=nan(1,256);
        h_r_max(KSN_max)=h_cum(KSN_max);
        subplot(2,2,3);
        plot(1:256,h_cum,'k',1:256,h_r,'--b','LineWidth',2);
        hold on
        stem(1:256,h_r_min,'r-->','LineWidth',2);
        stem(1:256,h_r_max,'r--<','LineWidth',2);
        axis([min(eye(:)) max(eye(:)) 1 WSz]);
        grid on;
        xlabel('Gray Level','FontWeight','Bold');
        ylabel('Rank','FontWeight','Bold');
        title('Variational row and borders of KSNnbh','FontWeight','Bold');
        set(gca,'FontWeight','Bold');
        drawnow
        hold off
        subplot(2,2,4);
        imshow(out);
        grid on;
        title(['KNV-neighborhood, K=' int2str(k)],'FontWeight','Bold');
        s=menu('Another K?','Yes','No'); 
    end
    %----------------------%ER-neighborhood--------------------------------
    case 3,
        r=1;
        RANKS=[5 10 20 40 80 120 160 200 250 300 350 400 500];
        VarRow=sort(eye(:));
        while r==1
            R_cntr=find(VarRow==centervalue);R_cntr=round(mean(R_cntr));
            
            ER=inputdlg({['ERplus in fraction of the fragment size',num2str(WSzX),'x',num2str(WSzY)],......
               ['ERminus in fraction of the fragment size',num2str(WSzX),'x',num2str(WSzY)] },'Enter ER',1,{'0.25','0.25'});
            ERpos=str2double(ER(1));
            ERplus=round(WSzX*WSzY*ERpos);
            ERneg=str2double(ER(2));
            ERminus=round(WSzX*WSzY*ERpos);
            Rleft=R_cntr-ERminus;if Rleft<1,Rleft=1;end
            LeftOrdStatist=VarRow(Rleft);
            Rright=R_cntr+ERplus;if Rright>WSz,Rright=WSz;end
            RightOrdStatist=VarRow(Rright);
            mask=(eye>=LeftOrdStatist).*(eye<=RightOrdStatist);
            out=eye.*mask;
            h_c=zeros(1,WSz);h_c(R_cntr)=centervalue;
            h_r=nan(1,WSz);h_r(Rright)=RightOrdStatist;
            h_l=nan(1,WSz);h_l(Rleft)=LeftOrdStatist;
            subplot(2,2,3);
            plot(1:WSz,VarRow,'k',1:WSz,h_c,'b--','LineWidth',2);
            hold on
            stem(1:WSz,h_l,'r-->','LineWidth',2);
            drawnow
            stem(1:WSz,h_r,'r--<','LineWidth',2);
            drawnow
            grid on;
            axis([1 WSz+ERplus 1 256]);
            xlabel('Pixel rank','FontWeight','Bold');
            ylabel('Order Statistics','FontWeight','Bold');
            title('Variat.row and borders of ERnbh','FontWeight','Bold');
            set(gca,'FontWeight','Bold');
            drawnow
            hold off
            subplot(2,2,4);
            imshow(out,[]);
            grid on;
            title(['ER-neighborhood, ERplus=', int2str(Rright-R_cntr),'; ERminus=',int2str(R_cntr-Rleft)],'FontWeight','Bold');
            r=menu('Another Rplus, Rminus?','Yes','No');
        end 
 %-------------------------Q-neighborhood-------------------------      
    case 4,
        r=1;
        VarRow=sort(eye(:));
        R_cntr=find(VarRow==centervalue);R_cntr=round(mean(R_cntr));
        while r==1,
            Rlr=inputdlg({['Left quantil in fraction of the fragment size',num2str(WSzX),'x',num2str(WSzY)],......
               ['Right quantil in fraction of the fragment size',num2str(WSzX),'x',num2str(WSzY)] },'Enter Quantil',1,{'0.05','0.95'});
            r_left=str2double(Rlr(1));
            Rleft=round(WSzX*WSzY*r_left);
            r_right=str2double(Rlr(2));
            Rright=round(WSzX*WSzY*r_right);
            LeftOrdStatist=VarRow(Rleft);
            RightOrdStatist=VarRow(Rright);
            mask=(eye>=LeftOrdStatist).*(eye<=RightOrdStatist);
            out=eye.*mask;
            h_c=zeros(1,WSz);h_c(R_cntr)=centervalue;
            h_r=nan(1,WSz);h_r(Rright)=RightOrdStatist;
            h_l=nan(1,WSz);h_l(Rleft)=LeftOrdStatist;
            subplot(2,2,3);
            plot(1:WSz,VarRow,'k',1:WSz,h_c,'b--','LineWidth',2);
            hold on
            stem(1:WSz,h_l,'r-->','LineWidth',2);
            drawnow
            stem(1:WSz,h_r,'r--<','LineWidth',2);
            drawnow
            grid on;
            axis([1 WSz 1 256]);
            xlabel('Pixel rank','FontWeight','Bold');
            ylabel('Order Statistics','FontWeight','Bold');
            title('Variat.row and borders of Qnbh','FontWeight','Bold');
            set(gca,'FontWeight','Bold');
            drawnow
            hold off
            subplot(2,2,4);
            imshow(out,[]);
            grid on;
            title(['ER-neighborhood, Qplus=', int2str(5*r_right),'%','; Qminus=',int2str(5*r_left),'%'],'FontWeight','Bold');
            r=menu('Other Rplus, Rminus?','Yes','No');
        end
  %-------------------------CL-neighborhood--------------------------------------------      
    case 5,
        r=1;
        h_sm=conv2(h,ones(1,5)/5,'same');
        while r==1,
            figure(2)
            plot(1:256,h_sm,'k-',1:256,h0,'b--','LineWidth',2);
            grid on;
            title('Histogram of the fragment','FontWeight','Bold');
            xlabel('Gray level','FontWeight','Bold');
            axis([min(eye(:)) max(eye(:)) 0 max(h)]);
            set(gca','FontWeight','Bold');
            drawnow;
            uiwait(msgbox('G-input: Select cluster left border on the histogram plot','Attention!','warn'));
            [Vleft,Hleft]=ginput(1);
            Vleft=round(Vleft);
            uiwait(msgbox('G-input: Select cluster right border on the histogram plot','Attention!','warn'));

            [Vright,Hright]=ginput(1);
            Vright=round(Vright);
            close(figure(2));
            mask=(eye>=Vleft).*(eye<=Vright);
            out=eye.*mask;
            h_left=nan(1,256);h_left(Vleft)=0.5*max(h_sm);
            h_right=nan(1,256);h_right(Vright)=0.5*max(h_sm);
            figure(1)
            subplot(2,2,3);
            plot(1:256,h_sm,'k-',1:256,h0,'b--','LineWidth',2);
            hold on;
            stem(1:256,h_left,'r-->','LineWidth',2);
            stem(1:256,h_right,'r--<','LineWidth',2);
            grid on;
            title('Smoothed window histogram and cluster borders','FontWeight','Bold');
            xlabel('Gray level','FontWeight','Bold');
            axis([min(eye(:))-5 max(eye(:))+5 0 max(h)]);
            set(gca','FontWeight','Bold');
            drawnow
            hold off
            subplot(2,2,4);
            imshow(out,[]);
            grid on;
            title(['Cl-neighborhood, Vleft=', int2str(Vleft),'; Vright=',int2str(Vright)],'FontWeight','Bold');
            r=menu('Other cluster borders?','Yes','No');
        end
    end
    s=menu('Another neighborhood?','Yes','No');
end
        
        

%--------------------------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
%----------------------------------------------------------------
function OUT=imghisto(INPIMG);

% Image histogram
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUT=imghisto(INPIMG);

[SzX SzY]=size(INPIMG);
histbuf=zeros(256,SzY);
for n=1:SzX
buf=INPIMG(n,:); buf=buf+256*([0:SzY-1]);
histbuf(buf+1)=histbuf(buf+1)+ones(1,SzY);
end
OUT=sum(histbuf'); 