function WienerFilter_CRC;

% Simulation of the ideal Wiener filter and an empirical Wiener filter
% for image deblurring and denoising
% PSNR-signal peak-to-noise std ratio
% BlurFactor - spread parameter of gaussian-shaped low-pass filter in frequency
% domain; roughly corresponds to fraction of the base band passed by the
% filter
% gamma is a weight parameter for the empirical Wiener filter design (values>1 are recommended)
% OUTPUT=[IMGNOISY IMGWIENER IMGEMPWIENER];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Call OUTIMG=WienerFilter_CRC;

% INPIMG=load_img_CRC;
load text256;
INPIMG=text256;
BF=[0.1,0.15,0.2,0.3,0.4,0.5,0.75,1,1000];
b=menu('Blur Factor','0.1','0.15','0.2','0.3','0.4','0.5','0.75','1','No blur');
BlurFactor=BF(b);
BlFactString=['Blurred and noisy image, BlurFactor=',num2str(BlurFactor)];
if BlurFactor>1,BlFactString=['Noisy image,no blur'];end

figure_n(6);
[SzX SzY]=size(INPIMG);ImgSz=min(SzX,SzY);
spimg=fft2(INPIMG);

%----------------------Formation of blurred image---------------------------
R=BlurFactor*(ImgSz^2)/8;
r=radius(ImgSz/2);r=r(1:ImgSz,1:ImgSz);
mask=exp(-(r.^2/(2*(R+eps).^2)));
mask=fftshift(mask)+eps;
spimg=spimg.*mask;
spimg2=abs(spimg).^2;
INPIMG_=real(ifft2(spimg));

%----------------Formation of additive noise interference------------------
NL=1;
while NL==1,
    psnr=[1,2,3,5,10,20,30,40,50,100];
    n=menu('Peak signal-to-noise ratio','1','2','3','5','10','20','30','40','50','100');
    PSNR=psnr(n);
    noise=(255/PSNR)*randn(size(INPIMG));
    spnoise=(abs(fft2(noise))).^2;
    N=(mean(mean(spnoise)));

    imgn=INPIMG_+noise;
    spimgn=fft2(imgn);
    spimgn2=abs(spimgn).^2;
    spimgn2=conv2(spimgn2,ones(7)/49,'same');


    figure(1);
    imshow(imgn,[]);
    title([BlFactString,'; PSNR=',num2str(PSNR)],'FontWeight','Bold');drawnow
    sigma=std2(INPIMG-imgn);
    figure(4);
    imshow(INPIMG-imgn,[]);
    title(['Image distortion std=',num2str(sigma,3)],'FontWeight','Bold');axis off;drawnow

% --------------Ideal Wiener filter----------------------------------------

    WIENER=spimg2./(spimg2+spnoise);
    WIENER=WIENER./mask;
    IMGWIENER=real(ifft2(spimgn.*WIENER));

    figure(2);
    imshow(IMGWIENER,[]);
    title('Ideal Wiener filter','FontWeight','Bold');
    axis off;
    drawnow
    [mn,sigma]=std2d(INPIMG-IMGWIENER);
    figure(5);
    imshow(INPIMG-IMGWIENER,[]);
    title(['Ideal Wiener filter restoration error, std=',num2str(sigma,3)],'FontWeight','Bold');
    axis off;
    drawnow

% --------------Empirical Wiener filter------------------------------------

    R=1;
    while R==1,
        gamma=menu('Select noise variance estimation factor','1','2','3');
        EMPWIENER=(spimgn2-gamma*N)./spimgn2;
        EMPWIENER=EMPWIENER.*(EMPWIENER>=0);
        IMGEMPWIENER=real(ifft2(spimgn.*(EMPWIENER./mask)));
        figure(3);
        imshow(IMGEMPWIENER,[]);
        title(['Empirical Wiener filter; Gamma=',num2str(gamma)],'FontWeight','Bold');
        axis off;
        drawnow
        [mn,sigma]=std2d(INPIMG-IMGEMPWIENER);
        figure(6);
        imshow(INPIMG-IMGEMPWIENER,[]);
        title(['Empir. Wiener filter restoration error, std=',num2str(sigma,3)],'FontWeight','Bold');
        axis off;
        drawnow
        R=menu('Another noise variance estimation factor?','Yes','No');
    end
    NL=menu('Another PSNR?','Yes','No');
end
close all
%------------------------------------------------------------
%------------------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
%--------------------------------------------------------------
function [mn,v]=std2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));
%----------------------------------------------------------------
function z=radius(R);

% Generating a function z=(X-X0)^2+(Y-Y0)^2; X0=Y0=R+1; 
% Resulting matrix size is (2R+1)*(2R+1);
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call z=radius(R);

X=(-R:R).^2;
X=kron(X,ones(2*R+1,1));

z=X+X';

