function demoire1_CRC;
% Filtering 1-D moire noise in INPIMG
% Automatic detection of noise components in the accumulated 1-D 
% column-wise-wise image power spectrum.
% Detectiont hreshold thr is a parameter for detecting moire peaks; 0<thr<1
% It represents an allowed increment of spectrum samples and
% is used for detection nonmonotonicity by checking the inequality:
% OUT(x+1)>(1+thr)*OUT(x), 
% Reccomended value 0.05
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call demoire1_CRC;

clf
figure_n(4);
load MoireNoiseImg;
INPIMG=MoireNoiseImg;
thr=inputdlg ({'Detection threshold='},'Enter detection threshold',1,{'0.05'});
thr=str2double(thr);
figure(1);
display1(INPIMG);
title('Input image','FontWeight','Bold');drawnow
[SzX SzY]=size(INPIMG);
INPIMG=INPIMG';
sp=fft(INPIMG);
sp2=abs(sp).^2;
sp1d=(sum(sp2')).^0.1;
%-------------------Spectrum smoothing and moire detection
sp1dsm=monotone(sp1d(1:SzX/2+1),thr);
mask(1:SzX/2+1)=sp1dsm<sp1d(1:SzX/2+1);
mask(2+SzX/2:SzX)=mask(SzX/2:-1:2);
mask=ones(size(sp1d))- mask;
mask_=kron(mask',ones(1,SzY));
%---------------------------------------------------------
figure(2);
plot(sp1d(1:SzX/2),'r','LineWidth',2);
axis tight;grid on; 
hold on
xlabel('Frequency index','FontWeight','Bold')
title('Av. power spectrum along rows');
drawnow
plot(sp1dsm,'b.','LineWidth',2);axis tight;
title(['Av. spectra of initial and filtered images; thr=',num2str(thr)],'FontWeight','Bold');
drawnow
set(gca,'FontWeight','Bold')
hold off
legend('Initial image spectrum','Filtered image spectrum')

OUTIMG=round((real(ifft(sp.*mask_))))';
figure(3)
display1(OUTIMG);
title('Filtered image','FontWeight','Bold');drawnow
figure(4);
dispnorm((OUTIMG-INPIMG')',4);
title('Noise removed','FontWeight','Bold');drawnow
%------------------------------------------------------
S=1;
while S==1,
    S=menu('Do you want to change detection threshold?', 'yes','no');
    if S==1,
    thr=inputdlg('Enter new detection threshold');
    thr=str2double(thr);
    sp1dsm=monotone(sp1d(1:SzX/2+1),thr);
    figure(2);
    plot(sp1d(1:SzX/2),'r','LineWidth',2);
    axis tight;grid on; 
    xlabel('Frequency index','FontWeight','Bold')
    set(gca,'FontWeight','Bold')
    hold on
    title('Av. power spectrum along rows','FontWeight','Bold');
    drawnow
    plot(sp1dsm,'b.','LineWidth',2);axis tight;
    title(['Av. power spectra along rows of initial and filtered images; thr=',num2str(thr),'FontWeight','Bold']);
    drawnow
    hold off
    mask(1:SzX/2+1)=sp1dsm<sp1d(1:SzX/2+1);
    mask(2+SzX/2:SzX)=mask(SzX/2:-1:2);
    mask=1-mask;
    mask_=kron(mask',ones(1,SzY));

    OUTIMG=round((real(ifft(sp.*mask_))))';
    figure(3);
    display1(OUTIMG);title('Filtered image','FontWeight','Bold');drawnow
    figure(4);
    dispnorm((OUTIMG-INPIMG')',4);
    title('Noise removed','FontWeight','Bold');drawnow
end
end

%---------------------------------------------------------------------
function OUT=monotone(curve,thr);

% Curve smoothing by monotone approximation
% Applicable to 1-D spectra smoothing
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUT=monotone(curve,thr);

SzX=max(size(curve));
OUT=curve;
for x=1:SzX-1;
	if 	OUT(x+1)>(1+thr)*OUT(x), 
		OUT(x+1)=OUT(x);
	end
end
%------------------------------------------------------------------

function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)
% Version July 15, 2001
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
close all;

switch N;
case 1, 
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
case 2, 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120);
   figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
case 3, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 4, 
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 5,
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-120)/2;
   figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30+2*H_Sz VerSz-75-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end

%----------------------------------------------------------
function display1(INPIMG)
% Displays a 2-D signal as an image 
% with normalization by gobal maximum and minimum
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call display1(INPIMG)

clf
colormap(gray(256))
Min=min(min(INPIMG)); Max=max(max(INPIMG));
if Max-Min==0, im=zeros(size(INPIMG)); else im=255*(INPIMG-Min)/(Max-Min); end

image(im);axis image;axis off;
%------------------------------------------------------------
function dspl=dispnorm(INPIMG,g)
% 2-D signal display with normalization by mean value and standard deviation
% g- is a parameter that defines ratio of the display range 0-255
% to the signal standard deviation 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 
% Call dispnorm(INPIMG,g)

[SzX SzY]=size(INPIMG);
Mean=mean(mean(INPIMG));		
stdv=sqrt(mean(mean((INPIMG-Mean).^2)))+eps;
dspl=round(255*(INPIMG-Mean)/(g*stdv))+128;
dspl=(dspl>=0).*dspl;
dspl=(dspl<=255).*dspl+255*ones(SzX,SzY).*(dspl>255);
image(dspl);axis image;axis off;



