
function impulse_noise_filtering_demo_CRC;
% Demo: Filtering impulse noise
% 
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il) 



load Mars;
t=Mars;
[SzX,SzY]=size(t);
[mn_0 Std_0]=std2d(t);
PP=menu('Select','Impulse Noise','"Pepper&Salt" Noise');
Perr=inputdlg({'Probability of error'},'Select',1,{'0.3'});
Perr=str2double(Perr);
% P=menu('Input probability of error','0.1','0.15','0.2','0.25','0.3','0.35','0.4','0.45','0.5');
% Perr=0.1+0.05*(P-1);
switch PP,
    case 1, r=rand(SzX,SzY)>Perr;t_r=t.*r+255*rand(size(t)).*(1-r);
    case 2, r=rand(SzX,SzY)>Perr;t_r=t.*r+255*(rand(size(t))>0.5).*(1-r);
end
figure_n(3);
figure(1);
imshow(t_r,[]);
title(['Distorted input image; Perr=',num2str(Perr)],'FontWeight','Bold');

f=menu('Select','Locations of distorted pixels are known','Locations of distorted pixels are not known');
switch f,
    case 1
    Rpt=1;
    while Rpt==1,
        Itr=menu('Input number of iteration','20','40','60','80','100','120','140','160','180','200');
        Itr=Itr*20;
        SS=sqrt(1-Perr);
        SzXX=round(SzX*SS);
        SzYY=round(SzX*SS);
% ---------------Generating circular spectral mask---------------------
        mask=zeros(SzX,SzY);
        Fr=round(SzY*SS);
        for k=1:Fr-1,
            L=round(sqrt(Fr^2-k^2+1));
            mask(k,1:L)=1;
        end
% ----------------------Main loop-------------------------------------  
        t_r_i=t_r.*r+128*(1-r);
        STDItr=zeros(1,Itr);
        for k=1:Itr,
            sp_dct=dct2(t_r_i);
            sp_dct_m=sp_dct.*mask;
            pr=t_r_i;
            t_r_i=idct2(sp_dct_m);
            t_r_i=srezka(t_r_i);
            t_r_i=t.*r+t_r_i.*(1-r);
            figure(2);
            imshow(t_r_i,[]);
            title(['Reconstructed image on ', num2str(k),'-th iteration'],'FontWeight','Bold');
            drawnow
        end
        OUTIMG_iter=srezka(t_r_i);
        OUTIMG3x3=t_r;
        for k=2:SzX-1,
            for l=2:SzY-1,
                if r(k,l)==0,OUTIMG3x3(k,l)=round(sum(sum(t_r(k-1:k+1,l-1:l+1).*r(k-1:k+1,l-1:l+1)))/sum(sum(r(k-1:k+1,l-1:l+1))));end
            end
        end
        figure(3)
        imshow(OUTIMG3x3,[]);
        title('3x3-filtered image','FontWeight','Bold');
        Rpt=menu('Repeat with other parameters?','Yes','No');
    end
    case 2,
        s=1;
        while s==1,
            Thr_i=menu('Iterative filter detection threshold','1.5','2','2.5','3','3.5');
            THR_i=1.5+0.5*(Thr_i-1);
            Niter=menu('Iterative filter number of iterations','5','10','15','20','25','30');
            Niter=5*Niter;
            Lapl=[-0.1 -0.15 -0.1;-0.15 1 -0.15;-0.1 -0.15 -0.1];
            OUTIMG_IterFilter=t_r;
            for t=1:Niter,
                Diff=conv2(OUTIMG_IterFilter,Lapl,'same');
                Diff=conv2(Diff,Lapl,'same');
                [mn, Std]=std2d(Diff);
                correct_mask=(abs(Diff-mn))>THR_i*Std;
                Smooth=conv2(OUTIMG_IterFilter,ones(3)/9,'same');
                OUTIMG_IterFilter=OUTIMG_IterFilter.*(1-correct_mask)+Smooth.*correct_mask;  
                figure(2)
                imshow(OUTIMG_IterFilter,[]);
                title(['Filtered image; Thr=',[num2str(THR_i)],'; Iter=',num2str(t)],'FontWeight','Bold');
                drawnow
            end
    %-------------Recursive filter-------------------------------
            figure(3)
            Thr=menu('Recursive filtering detection threshold','30','40','50','60');
            THR=30+10*(Thr-1);
            E=menu('Residual edge contrast','10','20','30','40','50');
            EC=10*E;
            OUTIMG_recurs=filtim_r_(t_r,THR,EC);
            s=menu('Repeat with other parameters?','Yes','No');
        end
end    
%-----------------------------------------------------------------
%-----------------------------------------------------------------

function OUTIMG=filtim_r_(INPIMG,thr,delta);

% Impulse noise filtering with a recursive algorithm 
% OUTIMG=INPIMG if INPIMG-conv(INPIMG,mask)<thr
% Otherwise OUTIMG=conv(INPIMG,mask)+delta*sign(INPIMG-conv(INPIMG,mask)); 
% thr - detection threshold;
% delta - a residual edge contrast for preserving edges;
% Call : OUTIMG=filtim_r(INPIMG,thr,delta);

[SzX SzY]=size(INPIMG);
OUTIMG=INPIMG;
% -----------------------First row-----------------------------------
for y=2:SzY,
	diff=INPIMG(1,y)-OUTIMG(1,y-1);
	signdiff=sign(diff);
	absdiff=abs(diff); 
	if absdiff>thr,OUTIMG(1,y)=OUTIMG(1,y-1)+delta*signdiff;end
end
%---------------------- Remaining rows-------------------------------
for x=2:SzX,
	for y=2:SzY-1,
        smth=0.3*(OUTIMG(x,y-1)+OUTIMG(x-1,y))+0.2*(OUTIMG(x-1,y-1)+OUTIMG(x-1,y+1));
        diff=INPIMG(x,y)-smth;
        signdiff=sign(diff);
        absdiff=abs(diff); 
        if absdiff>thr,    
            OUTIMG(x,y)=smth+delta*signdiff; 
        end
    end
    % Last pixel in row
    y=SzY;
	smth=0.35*(OUTIMG(x,y-1)+OUTIMG(x-1,y))+0.3*OUTIMG(x-1,y-1);
	diff=INPIMG(x,SzY)-smth;	
	signdiff=sign(diff);
	absdiff=abs(diff); 
    if absdiff>thr,   
        OUTIMG(x,SzY)=smth+delta*signdiff;
    end
    imshow(OUTIMG,[]);
    title(['Recursively filtered image, thr=',num2str(thr),'; delta=',num2str(delta)],'FontWeight','Bold');
    drawnow
end

%---------------------------------------------------------------
function figure_n(N)
% Generating N square figure windows of ScreenVerSz/2*ScreenVerSz pixels
% figure('Position',[HorPosition VerPosition HorSize VertSize]);
% Coordinate origin - left bottom corner of the screen
% Version July 15, 2001; Modified Oct. 2008, Tampere
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call figure_n(N)

% Generating a figure window
ScrSz=get(0,'ScreenSize');
HorSz=(ScrSz(3)-20); VerSz=(ScrSz(4));
% close all;

switch N;
case 1, close all
   Nvert=1;Nhor=1;H_Sz=(HorSz-80); V_Sz=(VerSz-120);
%    figure('Position',[30 VerSz-30-V_Sz V_Sz V_Sz]); colormap(gray(256))
    figure('Position',[80 VerSz-80-V_Sz V_Sz V_Sz]); colormap(gray(256))

case 2, close all 
   Nvert=1;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-200);
%    figure('Position',[30 VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
    figure('Position',[30 VerSz-200-H_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-H_Sz H_Sz H_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-200-H_Sz H_Sz V_Sz]);colormap(gray(256))

case 3,close all 
%    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-220)/2;
%    figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))

case 4,close all; 
%    Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-120)/2;
   Nvert=2;Nhor=2;H_Sz=(HorSz-80)/2; V_Sz=(VerSz-220)/2;
%    figure('Position',[30 VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz])

%    figure('Position',[30+H_Sz VerSz-30-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30 VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))

%    figure('Position',[30+H_Sz VerSz-2*V_Sz-75 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
case 5,
   close all
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-220)/2;
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[85+2*H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
case 6, 
   close all
   Nvert=2;Nhor=3;H_Sz=(HorSz-80)/3; V_Sz=(VerSz-220)/2;
   figure('Position',[30 VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[85+2*H_Sz VerSz-80-V_Sz H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[30 VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[60+H_Sz VerSz-2*V_Sz-180 H_Sz V_Sz]);colormap(gray(256))
   figure('Position',[80+2*H_Sz VerSz-180-2*V_Sz H_Sz V_Sz]);colormap(gray(256))
end
%------------------------------------------------------------------
function [mn,v]=std2d(INP);

% Standard deviation and mean of a 2-D array
% z=[mean stdeviation];
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call [mn v]=std2d(INP)

[SzX SzY]=size(INP);
mn=sum(sum(INP))/(SzX*SzY);
v=sqrt(sum(sum((INP-mn).^2))/(SzX*SzY));
%-------------------------                   ----------------
function OUTIMG=srezka(INPIMG);

% Image normalization by zeroing all pixels that are less then 0
% and assigning 255 to all pixels than exceed 255
% Copyright L. Yaroslavsky (yaro@eng.tau.ac.il)  
% Call OUTIMG=srezka(INPIMG);

OUTIMG=(INPIMG>0).*INPIMG;
OUTIMG=(OUTIMG<=255).*OUTIMG+255*(OUTIMG>255);
%---------------------------------------------------------------
% 